@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
-
 """
 Folium Element Module class IFrame
 ----------------------
 """
+
 import os
 
 import pytest
@@ -20,21 +19,21 @@ def test_create_empty_iframe():
 
 
 def test_create_iframe():
-    iframe = elem.IFrame(html='<p>test content<p>', width=60, height=45)
+    iframe = elem.IFrame(html="<p>test content<p>", width=60, height=45)
     iframe.render()
 
 
 @pytest.mark.headless
 def test_rendering_utf8_iframe():
-    iframe = elem.IFrame(html=u'<p>Cerrahpaşa Tıp Fakültesi</p>')
+    iframe = elem.IFrame(html="<p>Cerrahpaşa Tıp Fakültesi</p>")
 
     options = Options()
-    options.add_argument('-headless')
+    options.add_argument("-headless")
     driver = Firefox(options=options)
 
-    driver.get('data:text/html,' + iframe.render())
+    driver.get("data:text/html," + iframe.render())
     driver.switch_to.frame(0)
-    assert u'Cerrahpaşa Tıp Fakültesi' in driver.page_source
+    assert "Cerrahpaşa Tıp Fakültesi" in driver.page_source
 
 
 @pytest.mark.headless
@@ -45,18 +44,18 @@ def test_rendering_figure_notebook():
     elem.Html(text).add_to(figure.html)
     html = figure._repr_html_()
 
-    filepath = 'temp_test_rendering_figure_notebook.html'
+    filepath = "temp_test_rendering_figure_notebook.html"
     filepath = os.path.abspath(filepath)
-    with open(filepath, 'w') as f:
+    with open(filepath, "w") as f:
         f.write(html)
 
     options = Options()
-    options.add_argument('-headless')
+    options.add_argument("-headless")
     driver = Firefox(options=options)
     try:
-        driver.get('file://' + filepath)
+        driver.get("file://" + filepath)
         driver.switch_to.frame(0)
-        text_div = driver.find_element(By.CSS_SELECTOR, 'div')
+        text_div = driver.find_element(By.CSS_SELECTOR, "div")
         assert text_div.text == text
     finally:
         os.remove(filepath)
